function x3=solve_for_x1(rho,hh,z1,z2,fn)
%function x3=solve_for_x1(rho,hh,z1,z2,fn)
%----------------------------------------------------------
	% Function to solve a quartic polynomial equation by the 
	% method of bisection. The solution is the distance between
	% the shot and the first bounce of the multiple at the WB.
	% The equation is:
   	% (rho^2-1)x1^4-2(rho^2-1)hh*x1^3+[(rho^2-1)hh^2+rho^2*
	%	z2^2-z1^2]x1^2+2hh*z1^2*x1-z1^2*hh^2=0
	% Ax1^4+Bx1^3+Cx1^2+Dx1+E=0
	%----------------------------------------------------------

	% scale the input values by 1000
	hh_scl=hh/1000.;
	z1_scl=z1/1000.;
	z2_scl=z2/1000.;

	% compute function coefficients
	if (fn==1)
		A=(rho^2-1);
		B=-2*(rho^2-1)*hh_scl;
		C=(rho^2-1)*hh_scl^2+rho^2*z2_scl^2-z1_scl^2;
		D=2*hh_scl*z1_scl^2;
		E=-z1_scl^2*hh_scl^2;
	elseif (fn==2)
		A=4*(rho^2-1);
		B=-4*(rho^2-1)*hh_scl;
		C=(rho^2-1)*hh_scl^2+rho^2*z2_scl^2-4*z1_scl^2;
		D=4*hh_scl*z1_scl^2;
		E=-z1_scl^2*hh_scl^2;
	else
		fprintf(1,'ERROR: fn must be 1 or 2\n')
		return
	end

	% set initial guesses
	x1=0;
	x2=hh_scl;
	x3=(x1+x2)/2.;
	
	f1=A*x1^4+B*x1^3+C*x1^2+D*x1+E;
	f2=A*x2^4+B*x2^3+C*x2^2+D*x2+E;
	f3=A*x3^4+B*x3^3+C*x3^2+D*x3+E;

	% loop until solution found
	k=1;
	while(1)
		f1=A*x1^4+B*x1^3+C*x1^2+D*x1+E;
		f2=A*x2^4+B*x2^3+C*x2^2+D*x2+E;
		f3=A*x3^4+B*x3^3+C*x3^2+D*x3+E;
		if (abs(f3)<0.00001)
			x3=x3*1000.;
			return
       	end
       	if ((f1*f3)<0)
			x2=x3;
		else
			x1=x3;
		end
		x3=(x1+x2)/2;

		if (k>100)
			fprintf(1,'ERROR: could not solve for x1 in 1000 iters\n');
			return;
		end
		k=k+1;
	end
return
