module row_findnsmatch_mod
  
  use nhelix
  use ncnhconest1
  use ncnhconest2
	use row_ncnhconest
  use helicon
  use ncnhelicon
  
  use cgstep_mod
  use solver2_mod
  use verbosity_mod
  
  implicit none
  
contains

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !! Non-stationary matched filtering !!
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  !! Finds non-stationary matched-filter
  subroutine row_findnsmatch(data1,data2,targt,match1,match2,reg,mode,eps,mu)
    real, optional                        :: eps
    real, dimension(:), pointer           :: data1,data2,targt
    type (nfilter)                        :: match1,match2
    type ( filter)                        :: reg
    integer                               :: mode
	real								:: mu

    real, dimension(:), pointer :: coeffs
    integer :: stat,nhmax1,nhmax2,nitr,i
    logical :: sverb
    real    :: epsilon
	integer	:: ncoeffs1,ncoeffs2,nl

    sverb=(verb>2)
    nhmax1=size(match1%hlx(1)%lag)  ! assumes all filters are same length
    nhmax2=size(match2%hlx(1)%lag)  ! assumes all filters are same length

    nitr=size(match1%hlx(1)%lag)+size(match2%hlx(1)%lag)-1
    epsilon=1.
    if (present(eps)) epsilon=eps

    write(0,*) 'epsilon = ',epsilon

	ncoeffs1=nhmax1*size(match1%hlx)
	ncoeffs2=nhmax2*size(match2%hlx)

    allocate (coeffs(ncoeffs1+ncoeffs2))
    coeffs=0.

    ! Regularized solver
    if (verb>2) write(0,*) '  using regularized conjugate gradients'
    call helicon_init(reg)                       ! initialize regularizer
    call row_ncnhconest_init(data1,data2,match1,match2,nhmax1,nhmax2,&
		& ncoeffs1,ncoeffs2,mu) ! initialize operators

    ! solve problem
    call solver_reg(row_ncnhconest_lop,cgstep,helicon_lop,size(coeffs), &
          		&   coeffs,targt,nitr,epsilon,x0=coeffs,verb=sverb)

    !! Copy coefficients into NS filter type
    do i=1,size(match1%hlx)
       match1%hlx(i)%flt(1:nhmax1)=coeffs(1+(i-1)*nhmax1:i*nhmax1)
    end do
	nl=size(match1%hlx)*nhmax1 !! last coeff from match1
    do i=1,size(match2%hlx)
       match2%hlx(i)%flt(1:nhmax2)=coeffs(nl+1+(i-1)*nhmax2:i*nhmax2)
    end do

    deallocate(coeffs)
    call cgstep_close()

  end subroutine row_findnsmatch

	subroutine applynsmatch(input,output,match)
    real, dimension(:), pointer :: input, output
    type (nfilter)              :: match
    integer stat
    call ncnhelicon_init(match)
    stat=ncnhelicon_lop(.false.,.false.,input,output)
    call ncnhelicon_close()
  end subroutine applynsmatch

end module row_findnsmatch_mod
